@@ -351,7 +351,7 @@ purple_conversation_new(PurpleConversationType type, PurpleAccount *account,
 		}
 
 		/*
-		 * This hack is necessary because some prpls (MSN) have unnamed chats
+		 * This hack is necessary because some prpls have unnamed chats
 		 * that all use the same name.  A PurpleConversation for one of those
 		 * is only ever re-used if the user has left, so calls to
 		 * purple_conversation_new need to fall-through to creating a new
@@ -1551,6 +1551,14 @@ purple_conv_chat_write(PurpleConvChat *chat, const char *who, const char *messag
 	if (purple_conv_chat_is_user_ignored(chat, who))
 		return;
 
+	if (mtime < 0) {
+		purple_debug_error("conversation",
+				"purple_conv_chat_write ignoring negative timestamp\n");
+		/* TODO: Would be more appropriate to use a value that indicates
+		   that the timestamp is unknown, and surface that in the UI. */
+		mtime = time(NULL);
+	}
+
 	if (!(flags & PURPLE_MESSAGE_WHISPER)) {
 		const char *str;
 
@@ -1978,7 +1986,6 @@ purple_conv_chat_clear_users(PurpleConvChat *chat)
 	}
 
 	g_hash_table_remove_all(chat->users);
-	chat->users = NULL;
 
 	g_list_free(users);
 	chat->in_room = NULL;
