@@ -563,7 +563,6 @@ aim_auth_securid_send(OscarData *od, const char *securid)
 {
 	FlapConnection *conn;
 	FlapFrame *frame;
-	aim_snacid_t snacid;
 	int len;
 
 	if (!od || !(conn = flap_connection_getbytype_all(od, SNAC_FAMILY_AUTH)) || !securid)
@@ -573,7 +572,7 @@ aim_auth_securid_send(OscarData *od, const char *securid)
 
 	frame = flap_frame_new(od, 0x02, 10+2+len);
 
-	snacid = aim_cachesnac(od, SNAC_FAMILY_AUTH, SNAC_SUBTYPE_AUTH_SECURID_RESPONSE, 0x0000, NULL, 0);
+	/* aim_snacid_t snacid = */ aim_cachesnac(od, SNAC_FAMILY_AUTH, SNAC_SUBTYPE_AUTH_SECURID_RESPONSE, 0x0000, NULL, 0);
 	aim_putsnac(&frame->data, SNAC_FAMILY_AUTH, SNAC_SUBTYPE_AUTH_SECURID_RESPONSE, 0);
 
 	byte_stream_put16(&frame->data, len);
